#ifndef ZP_ASSEMBLY_GENERATOR_H_
#define ZP_ASSEMBLY_GENERATOR_H_

#include "Optimizer.h"
#include "asmjit/src/asmjit/x86.h"

namespace zp
{
    class IRVisitor;

    class AssemblyGenerator;
}


class zp::IRVisitor
{
public:
    virtual void visitBlockInstruction(const BlockInstruction&) = 0;
    virtual void visitUnionBlockInstruction(const UnionBlockInstruction&) = 0;
    virtual void visitIOInstruction(const IOInstruction&) = 0;
    virtual void visitSimpleBlockInstruction(const SimpleBlockInstruction&) = 0;
    virtual void visitLinearLoopInstruction(const LinearLoopInstruction&) = 0;
    virtual void visitLoopInstruction(const LoopInstruction&) = 0;
};


class zp::AssemblyGenerator :
    public IRVisitor
{
    asmjit::X86Assembler as;
public:

    inline AssemblyGenerator(asmjit::CodeHolder* ch) :
        as{ ch } {}

    virtual void visitBlockInstruction(const BlockInstruction&);
    virtual void visitUnionBlockInstruction(const UnionBlockInstruction&);
    virtual void visitIOInstruction(const IOInstruction&);
    virtual void visitSimpleBlockInstruction(const SimpleBlockInstruction&);
    virtual void visitLinearLoopInstruction(const LinearLoopInstruction&);
    virtual void visitLoopInstruction(const LoopInstruction&);
};




#endif // ZP_ASSEMBLY_GENERATOR_H_
